==> ockam_transport_tcp-0.29.0.crate: ockam_transport_tcp-0.29.0/src/workers/worker_pair.rs <==
use crate::{
    atomic::{self, ArcBool},
    TcpRecvProcessor, TcpSendWorker,
};
use ockam_core::{Address, Result};
use ockam_node::Context;
use ockam_transport_core::TransportError;
use std::net::SocketAddr;
use tokio::net::TcpStream;
use tracing::{debug, trace};

/// Transmit and receive peers of a TCP connection
#[derive(Debug)]
pub(crate) struct WorkerPair {
    hostnames: Vec<String>,
    peer: SocketAddr,
    tx_addr: Address,
    _rx_addr: Address,
    _run: ArcBool,
}

impl WorkerPair {
    pub fn hostnames(&self) -> &Vec<String> {
        &self.hostnames
    }
    pub fn peer(&self) -> SocketAddr {
        self.peer
    }
    pub fn tx_addr(&self) -> Address {
        self.tx_addr.clone()
    }
}

impl WorkerPair {
    pub(crate) async fn new_with_stream(
        ctx: &Context,
        stream: TcpStream,
        peer: SocketAddr,
        hostnames: Vec<String>,
    ) -> Result<Self> {
        trace!("Creating new worker pair from stream");

        // TODO: Should we also check for TcpTransport run flag?
        let run = atomic::new(true);

        let tx_addr = Address::random(0);
        let rx_addr = Address::random(0);

        // Create two workers based on the split TCP I/O streams
        let (rx, tx) = stream.into_split();
        let sender = TcpSendWorker::new(tx, peer);
        let receiver =
            TcpRecvProcessor::new(rx, run.clone(), format!("{}#{}", crate::TCP, peer).into());

        // Derive local worker addresses, and start them
        ctx.start_worker(tx_addr.clone(), sender).await?;
        ctx.start_processor(rx_addr.clone(), receiver).await?;

        // Return a handle to the worker pair
        Ok(WorkerPair {
            hostnames,
            peer,
            tx_addr,
            _rx_addr: rx_addr,
            _run: run,
        })
    }

    pub(crate) async fn start(
        ctx: &Context,
        peer: SocketAddr,
        hostnames: Vec<String>,
    ) -> Result<Self> {
        debug!("Starting worker connection to remote {}", peer);

        // TODO: make i/o errors into ockam_error
        let stream = TcpStream::connect(peer)
            .await
            .map_err(TransportError::from)?;
        Self::new_with_stream(ctx, stream, peer, hostnames).await
    }
}

==> ockam_transport_tcp-0.31.0.crate: ockam_transport_tcp-0.31.0/src/workers/worker_pair.rs <==
use crate::{
    atomic::{self, ArcBool},
    TcpRecvProcessor, TcpSendWorker,
};
use ockam_core::{Address, Result};
use ockam_node::Context;
use ockam_transport_core::TransportError;
use std::net::SocketAddr;
use tokio::net::TcpStream;
use tracing::{debug, trace};

/// Transmit and receive peers of a TCP connection
#[derive(Debug)]
pub(crate) struct WorkerPair {
    hostnames: Vec<String>,
    peer: SocketAddr,
    tx_addr: Address,
    _rx_addr: Address,
    _run: ArcBool,
}

impl WorkerPair {
    pub fn hostnames(&self) -> &[String] {
        &self.hostnames
    }
    pub fn peer(&self) -> SocketAddr {
        self.peer
    }
    pub fn tx_addr(&self) -> Address {
        self.tx_addr.clone()
    }
}

impl WorkerPair {
    pub(crate) async fn new_with_stream(
        ctx: &Context,
        stream: TcpStream,
        peer: SocketAddr,
        hostnames: Vec<String>,
    ) -> Result<Self> {
        trace!("Creating new worker pair from stream");

        // TODO: Should we also check for TcpTransport run flag?
        let run = atomic::new(true);

        let tx_addr = Address::random(0);
        let rx_addr = Address::random(0);

        // Create two workers based on the split TCP I/O streams
        let (rx, tx) = stream.into_split();
        let sender = TcpSendWorker::new(tx, peer);
        let receiver =
            TcpRecvProcessor::new(rx, run.clone(), format!("{}#{}", crate::TCP, peer).into());

        // Derive local worker addresses, and start them
        ctx.start_worker(tx_addr.clone(), sender).await?;
        ctx.start_processor(rx_addr.clone(), receiver).await?;

        // Return a handle to the worker pair
        Ok(WorkerPair {
            hostnames,
            peer,
            tx_addr,
            _rx_addr: rx_addr,
            _run: run,
        })
    }

    pub(crate) async fn start(
        ctx: &Context,
        peer: SocketAddr,
        hostnames: Vec<String>,
    ) -> Result<Self> {
        debug!("Starting worker connection to remote {}", peer);

        // TODO: make i/o errors into ockam_error
        let stream = TcpStream::connect(peer)
            .await
            .map_err(TransportError::from)?;
        Self::new_with_stream(ctx, stream, peer, hostnames).await
    }
}
